<?php

namespace Tests\Unit\Controllers;

use App\Rules\StrAlphaUnderscore;
use App\Rules\StrLowercase;
use App\Rules\URL\KeywordBlacklist;
use App\Url;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Tests\TestCase;

class UrlControllerTest extends TestCase
{
    /**
     * The user_id column in the table of Url must be null.
     *
     * @test
     * @group u-controller
     */
    public function shortenUrl_user_id()
    {
        $longUrl = 'https://laravel.com';

        $this->post(route('createshortlink'), [
            'long_url' => $longUrl,
        ]);

        $url = Url::whereLongUrl($longUrl)->first();

        $this->assertSame(null, $url->user_id);
    }

    /**
     * The user_id column in the table of Url must be filled in with the value
     * of the authenticated user id.
     *
     * @test
     * @group u-controller
     */
    public function shortenUrl_user_id_2()
    {
        $user = $this->admin();
        $longUrl = 'https://laravel.com';

        $this->loginAsAdmin();

        $this->post(route('createshortlink'), [
            'long_url' => $longUrl,
        ]);

        $url = Url::whereLongUrl($longUrl)->first();

        $this->assertSame($user->id, $url->user_id);
    }

    /**
     * The keyword column in the table of Url must be filled with a value that
     * generated by UrlHub. The is_custom column must be filled with 0 / false.
     *
     * @test
     * @group u-controller
     */
    public function shortenUrl_not_custom()
    {
        $longUrl = 'https://laravel.com';

        $this->post(route('createshortlink'), [
            'long_url' => $longUrl,
        ]);

        $url = Url::whereLongUrl($longUrl)->first();

        $this->assertEquals(
            uHub('hash_length'),
            strlen($url->keyword)
        );
        $this->assertFalse($url->is_custom);
    }

    /**
     * The keyword column in the table of Url must be filled in with the value
     * given by User. The is_custom column must be filled with 1 / true.
     *
     * @test
     * @group u-controller
     */
    public function shortenUrl_is_custom()
    {
        config()->set('urlhub.hash_length', 6);

        $longUrl = 'https://laravel.com';
        $customKey = 'foo_bar';

        $this->post(route('createshortlink'), [
            'long_url'   => $longUrl,
            'custom_key' => $customKey,
        ]);

        $url = Url::whereLongUrl($longUrl)->first();
        $this->assertSame($customKey, $url->keyword);
        $this->assertTrue($url->is_custom);
    }

    /**
     * @test
     * @group u-controller
     */
    public function custom_key_validation_pass()
    {
        $response = $this->post(route('home').'/validate-custom-key', [
            'keyword' => 'hello',
        ]);

        $response->assertJson(['success' => 'Available']);
    }

    /**
     * @test
     * @group u-controller
     * @dataProvider customKeyValidation_fail
     */
    public function custom_key_validation_fail($data)
    {
        factory(Url::class)->create([
            'keyword' => 'laravel',
        ]);

        $request = new Request;

        $v = Validator::make($request->all(), [
            'keyword' => [
                'max:20',
                'unique:urls',
                new StrAlphaUnderscore,
                new StrLowercase,
                new KeywordBlacklist,
            ],
        ]);

        $response = $this->post(route('home').'/validate-custom-key', [
            'keyword' => $data,
        ]);

        $response->assertJson(['errors' => $v->errors()->all()]);
    }

    public function customKeyValidation_fail()
    {
        return [
            [str_repeat('a', 50)],
            ['FOOBAR'],
            ['foo-bar'],
            ['foo~bar'],
            ['login'],
        ];
    }
}
